/**
 * Copyright (c) 2008 - 2013 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */
import com.smaxe.uv.media.core.VideoFrame;
import com.smaxe.uv.media.java.swing.JVideoScreen;
import com.smaxe.uv.media.player.IMediaPlayerModel;
import com.smaxe.uv.media.player.JMediaPlayer;
import com.smaxe.uv.media.sdk.IMediaSdk;
import com.smaxe.uv.media.sdk.MediaSdkFactory;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/**
 * <code>ExMediaSdkMediaPlayerPlayLocalFile</code> - {@link IMediaSdk} MediaPlayer example
 * to play local media file.
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-media-sdk" target="_blank">JUV Media SDK</a>
 */
public final class ExMediaSdkMediaPlayerPlayLocalFile extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // Evaluation version has some limitations:
        // - video decoder watermarks frames (and blacks it sometimes)
        
        final String file = "sample.mp4";
        
        IMediaSdk mediaSdk = MediaSdkFactory.getMediaSdk();
        
        playMedia(mediaSdk, mediaSdk.createMediaPlayerModelSdk().create(file));
    }
    
    /**
     * Plays the media.
     * 
     * @param mediaSdk
     * @param model media player model
     * @throws Exception if an exception occurred
     */
    public static void playMedia(final IMediaSdk mediaSdk, final IMediaPlayerModel model) throws Exception
    {
        final JFrame frame = new JFrame();
        
        final JVideoScreen videoScreen = new JVideoScreen();
        
        final JMediaPlayer mediaPlayer = JMediaPlayer.create(model);
        
        mediaPlayer.setListener(new JMediaPlayer.ListenerAdapter()
        {
            @Override
            public void onVideoFrame(final VideoFrame frame, final int[] frameCropOffset)
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                        videoScreen.setFrameCropOffset(frameCropOffset);
                        
                        videoScreen.setFrame(frame);
                    }
                });
            }
        });
        
        JButton start = new JButton(new AbstractAction("Play")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                mediaPlayer.startPlayback();
            }
        });
        
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(videoScreen, BorderLayout.CENTER);
        frame.getContentPane().add(start, BorderLayout.SOUTH);
        frame.pack();
        
        frame.setVisible(true);
    }
    
    /**
     * Constructor.
     */
    private ExMediaSdkMediaPlayerPlayLocalFile()
    {
    }
}