/**
 * Copyright (c) 2008 - 2017 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.logger.ILogger;
import com.smaxe.logger.support.Loggers;
import com.smaxe.uv.client.rtsp.IVideo;
import com.smaxe.uv.client.rtsp.video.AbstractVideo;
import com.smaxe.uv.client.rtsp.video.DispatcherVideo;
import com.smaxe.uv.client.x.RtspPlayer;
import com.smaxe.uv.media.audio.AsyncAudioPlayback;
import com.smaxe.uv.media.core.AudioFrame;
import com.smaxe.uv.media.core.AudioTrackInfo;
import com.smaxe.uv.media.core.IAudioDecoder;
import com.smaxe.uv.media.core.IAudioPlayback;
import com.smaxe.uv.media.core.IVideoDecoder;
import com.smaxe.uv.media.core.MediaTrackInfo;
import com.smaxe.uv.media.core.VideoFrame;
import com.smaxe.uv.media.java.audio.JavaAudioPlayback;
import com.smaxe.uv.media.java.swing.JVideoScreen;
import com.smaxe.uv.media.sdk.IMediaSdk;
import com.smaxe.uv.media.sdk.MediaSdkFactory;
import com.smaxe.uv.stream.MediaData;
import com.smaxe.uv.stream.MediaDataFactory;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

/**
 * <code>ExMediaSdkPlayRealtimeRtspStream</code> - {@link IMediaSdk} example that plays real-time RTSP/RTP stream.
 * <p> Note:
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-media-sdk" target="_blank">JUV Media SDK</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtsp-client" target="_blank">JUV RTSP/RTP Client</a>
 */
public final class ExMediaSdkPlayRealtimeRtspStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // Evaluation version has some limitations:
        // - video decoder watermarks frames
        
        final String url = args.length == 0 ? "rtsp://184.72.239.149/vod/mp4:BigBuckBunny_175k.mov" : args[0];
        
        final IAudioPlayback audioPlayback = new AsyncAudioPlayback(new JavaAudioPlayback());
        final RtspPlayer player = new RtspPlayer();
        
        final IMediaSdk mediaSdk = MediaSdkFactory.getMediaSdk();
        
        final JVideoScreen videoScreen = new JVideoScreen();
        
        final JButton start = new JButton(new AbstractAction("Play")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                new Thread(new Runnable()
                {
                    public void run()
                    {
                        player.setLogger(Loggers.createSoLogger(ILogger.INFO, "RtspPlayer"));
                        
                        player.setMediaDataTimeout(5000 /*milliseconds*/);
                        player.setReconnectOnMediaEnd(true /*reconnect*/);
                        player.setReconnectOnMediaTimeout(true /*reconnect*/);
                        // define either TCP (false) or UDP mode (true)
                        player.setUseUdpTransport(false /*useUdpTransport*/);
                        
                        player.play(url, new DispatcherVideo(new Video(mediaSdk, videoScreen, audioPlayback)));
                    }
                }, "ExMediaSdkPlayRealtimeRtspStream").start();
            }
        });
        
        final JButton stop = new JButton(new AbstractAction("Stop")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                player.close();
            }
        });
        
        JPanel buttons = new JPanel(new FlowLayout());
        
        buttons.add(start);
        buttons.add(stop);
        
        final JFrame frame = new JFrame();
        
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(videoScreen, BorderLayout.CENTER);
        frame.getContentPane().add(buttons, BorderLayout.SOUTH);
        frame.pack();
        
        frame.setTitle(url);
        frame.setVisible(true);
    }
    
    /**
     * <code>Video</code>
     */
    private final static class Video extends AbstractVideo implements IVideo
    {
        // fields
        private final IMediaSdk mediaSdk;
        private final JVideoScreen videoScreen;
        private IAudioPlayback audioPlayback;
        
        // state
        private IAudioDecoder audioDecoder;
        private IVideoDecoder videoDecoder;
        
        /**
         * Constructor.
         * 
         * @param mediaSdk
         * @param videoScreen
         */
        public Video(final IMediaSdk mediaSdk, final JVideoScreen videoScreen, final IAudioPlayback audioPlayback)
        {
            this.mediaSdk = mediaSdk;
            this.videoScreen = videoScreen;
            this.audioPlayback = audioPlayback;
        }
        
        // IVideo implementation
        
        @Override
        public void onMediaData(final MediaTrackInfo track, final MediaData data)
        {
            System.out.println("onMediaData: " + track + "  " + data);
            
            switch (track.type)
            {
                case MediaTrackInfo.AUDIO:
                {
                    if (audioDecoder == null)
                    {
                        audioDecoder = mediaSdk.getMediaCodecSdk().createAudioDecoder(track.codec, track.codecConfig, null);
                    }
                    
                    audioDecoder.decodeFrame(data.timestamp, MediaDataFactory.getMediaDataPayload(data), new IAudioDecoder.ICallback()
                    {
                        public void onDecodedFrame(final AudioFrame frame)
                        {
                            if (!audioPlayback.isStarted())
                            {
                                try
                                {
                                    audioPlayback.start(new AudioTrackInfo((int) (frame.rate), frame.getChannels()), 32 * 1024);
                                }
                                catch (Exception e)
                                {
                                    e.printStackTrace();
                                }
                            }
                            
                            try
                            {
                                audioPlayback.write(frame.pcm);
                            }
                            catch (Exception e)
                            {
                                e.printStackTrace();
                            }
                        }
                        
                        public void onException(Object tag, Exception e)
                        {
                            e.printStackTrace();
                        }
                    });
                    
                } break;
                case MediaTrackInfo.VIDEO:
                {
                    if (videoDecoder == null)
                    {
                        videoDecoder = mediaSdk.getMediaCodecSdk().createVideoDecoder(track.codec, track.codecConfig, null);
                    }
                    
                    videoDecoder.decodeFrame(data.timestamp, MediaDataFactory.getMediaDataPayload(data), new IVideoDecoder.CallbackAdapter()
                    {
                        @Override
                        public void onDecodedFrame(final VideoFrame frame)
                        {
                            SwingUtilities.invokeLater(new Runnable()
                            {
                                public void run()
                                {
                                    videoScreen.setFrame(frame);
                                }
                            });
                        }
                        
                        @Override
                        public void onException(final Object tag, final Exception e)
                        {
                            e.printStackTrace();
                        }
                    });
                } break;
            }
        }
        
        @Override
        public void release()
        {
            if (audioDecoder != null)
            {
                audioDecoder.release();
                audioDecoder = null;
            }
            
            if (videoDecoder != null)
            {
                videoDecoder.release();
                videoDecoder = null;
            }
        }
    }
    
    /**
     * Constructor.
     */
    private ExMediaSdkPlayRealtimeRtspStream()
    {
    }
}