/**
 * Copyright (c) 2008 - 2017 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */
import com.smaxe.logger.ILogger;
import com.smaxe.logger.support.Loggers;
import com.smaxe.uv.client.rtsp.IVideo;
import com.smaxe.uv.client.rtsp.video.AbstractVideo;
import com.smaxe.uv.client.rtsp.video.DispatcherVideo;
import com.smaxe.uv.client.x.RtspPlayer;
import com.smaxe.uv.media.core.IMediaWriter;
import com.smaxe.uv.media.core.MediaContainer;
import com.smaxe.uv.media.core.MediaTrackInfo;
import com.smaxe.uv.media.java.swing.JVideoScreen;
import com.smaxe.uv.media.sdk.IMediaSdk;
import com.smaxe.uv.media.sdk.MediaSdkFactory;
import com.smaxe.uv.stream.MediaData;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * <code>ExMediaSdkRecordRtspStream</code> - {@link IMediaSdk} example that records RTSP/RTP stream.
 * <p> Note:
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-media-sdk" target="_blank">JUV Media SDK</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtsp-client" target="_blank">JUV RTSP/RTP Client</a>
 */
public final class ExMediaSdkRecordRtspStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // Evaluation version has some limitations:
        // - video decoder watermarks frames (and blacks it sometimes)
        
        final String url = args.length == 0 ? "rtsp://184.72.239.149/vod/mp4:BigBuckBunny_175k.mov" : args[0];
        
        final RtspPlayer player = new RtspPlayer();
        
        player.setLogger(Loggers.createSoLogger(ILogger.INFO, "RtspPlayer"));
        
        final IMediaSdk mediaSdk = MediaSdkFactory.getMediaSdk();
        
        final JVideoScreen videoScreen = new JVideoScreen();
        
        final JButton start = new JButton(new AbstractAction("Record")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                new Thread(new Runnable()
                {
                    public void run()
                    {
                        player.setLogger(Loggers.createSoLogger(ILogger.INFO, "RtspPlayer"));
                        
                        // define either TCP (false) or UDP mode (true)
                        player.setUseUdpTransport(false);
                        
                        player.play(url, new DispatcherVideo(new Video(mediaSdk)));
                    }
                }, "ExMediaSdkRecordRtspStream").start();
            }
        });
        
        final JButton stop = new JButton(new AbstractAction("Stop")
        {
            private final static long serialVersionUID = -3538347901572402162L;
            
            public void actionPerformed(final ActionEvent e)
            {
                player.close();
            }
        });
        
        JPanel buttons = new JPanel(new FlowLayout());
        
        buttons.add(start);
        buttons.add(stop);
        
        final JFrame frame = new JFrame();
        
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(videoScreen, BorderLayout.CENTER);
        frame.getContentPane().add(buttons, BorderLayout.SOUTH);
        frame.pack();
        
        frame.setTitle(url);
        frame.setVisible(true);
    }
    
    /**
     * <code>Video</code>
     */
    private final static class Video extends AbstractVideo implements IVideo
    {
        // fields
        private final IMediaSdk mediaSdk;
        
        // state
        private IMediaWriter writer = null;
        
        /**
         * Constructor.
         * 
         * @param mediaSdk
         */
        public Video(final IMediaSdk mediaSdk)
        {
            this.mediaSdk = mediaSdk;
        }
        
        // IVideo implementation
        
        @Override
        public void onMediaData(final MediaTrackInfo track, final MediaData data)
        {
            System.out.println("onMediaData: " + data);
            
//            if (true) return;
            
            try
            {
                if (writer == null)
                {
                    writer = createWriter(MediaContainer.MKV, new File("e:/h264_field005.mkv"));
                }
                
                writer.writeTrackSample(track, data);
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
        
        @Override
        public void release()
        {
            try
            {
                if (writer != null)
                {
                    writer.close();
                }
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
        
        /**
         * Creates {@link IMediaWriter}.
         * 
         * @param container
         * @param file
         * @return media writer
         * @throws IOException if an I/O exception occurred
         */
        private IMediaWriter createWriter(final MediaContainer container, final File file) throws IOException
        {
            Map<String, Object> properties = new HashMap<String, Object>();
            
            properties.put(IMediaWriter.Property.APP, "My App");
            properties.put(IMediaWriter.Property.CREATION_TIME, new java.util.Date());
            properties.put(IMediaWriter.Property.TITLE, "My Recording");
            
            return mediaSdk.getMediaContainerSdk().openFileWriter(file, container, properties);
        }
    }
    
    /**
     * Constructor.
     */
    private ExMediaSdkRecordRtspStream()
    {
    }
}